package com.ningdatech.pmapi.organization.controller;

import com.ningdatech.basic.model.PageVo;
import com.wflow.bean.dto.WflowOrgProcdefDto;
import com.wflow.bean.entity.WflowOrgModelHistorys;
import com.wflow.bean.vo.OrgProcdefVo;
import com.wflow.service.OrgProcdefService;
import com.wflow.workflow.service.OrgProcessModelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Classname OrganizationProcessController
 * @Description
 * @Date 2023/1/13 11:13
 * @Author PoffyZhang
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/organization/procdef")
@Api(value = "OrganizationProcdef", tags = "单位配置-流程配置")
@RequiredArgsConstructor
public class OrganizationProcdefController {

    private final OrgProcdefService orgProcdefService;

    private final OrgProcessModelService orgProcessModelService;

    @ApiOperation(value = "单位流程配置列表", notes = "单位流程配置列表")
    @GetMapping("/list")
    public PageVo<OrgProcdefVo> list(@RequestParam String orgCode, @RequestParam(defaultValue = "1") Integer pageNum,
                                            @RequestParam(defaultValue = "10") Integer pageSize) {
        return orgProcdefService.getOrgProcessByOrgCode(orgCode,pageNum,pageSize);
    }

    @ApiOperation(value = "单位流程配置详情", notes = "单位流程配置详情")
    @GetMapping("/detail/{procdefId}")
    public OrgProcdefVo detail(@PathVariable String procdefId) {
        WflowOrgModelHistorys lastVersionModel = orgProcessModelService.getLastVersionModel(procdefId);
        OrgProcdefVo vo = new OrgProcdefVo();
        BeanUtils.copyProperties(lastVersionModel,vo);
        return vo;
    }

    @ApiOperation(value = "单位流程配置保存", notes = "单位流程配置保存")
    @PostMapping("/save")
    public String save(@Validated @RequestBody WflowOrgProcdefDto wflowOrgProcdefDto) {
        return orgProcessModelService.saveProcess(wflowOrgProcdefDto);
    }

    @ApiOperation(value = "单位流程配置部署", notes = "单位流程配置部署")
    @PostMapping("/deploy/{processDefId}")
    public String deploy(@Validated @PathVariable String processDefId) {
        return orgProcessModelService.deployProcess(processDefId);
    }

    @ApiOperation(value = "单位流程配置删除", notes = "单位流程配置删除")
    @PostMapping("/delete/{procdefId}")
    public Boolean delete(@PathVariable String procdefId) {
        return orgProcessModelService.delProcess(procdefId);
    }
}
