package com.ningdatech.pmapi.todocenter.manage;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.pmapi.common.constant.ProjectDeclareConstants;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.common.util.ExcelExportStyle;
import com.ningdatech.pmapi.todocenter.enums.ProcessStatusEnum;
import com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProjectListExportDTO;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.workflow.bean.dto.ProcessInstanceOwnerDto;
import com.wflow.workflow.bean.process.OrgUser;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.bean.vo.ProcessTaskVo;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.UserDeptOrLeaderService;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.repository.ProcessDefinition;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskQuery;

import org.springframework.stereotype.Component;
import com.ningdatech.basic.model.PageVo;

import com.ningdatech.pmapi.todocenter.model.dto.req.ReqToBeProcessedDTO;
import com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProcessedDTO;
import lombok.RequiredArgsConstructor;


import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author CMM
 * @since 2023/01/12 16:09
 */
@Component
@RequiredArgsConstructor
public class TodoCenterManage {

    private final TaskService taskService;
    private final RepositoryService repositoryService;
    private final RuntimeService runtimeService;
    private final UserDeptOrLeaderService userDeptOrLeaderService;
    private final ProcessInstanceService processService;
    public PageVo<ResToBeProcessedDTO> queryProjectList(ReqToBeProcessedDTO param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取入参分页信息
        Integer pageSize = param.getPageSize();
        Integer pageNumber = param.getPageNumber();

        TaskQuery taskQuery = taskService.createTaskQuery();
        taskQuery.active().taskCandidateOrAssigned(String.valueOf(userId)).orderByTaskCreateTime().desc();
        Page<ResToBeProcessedDTO> page = param.page();

        List<Task> taskList = taskQuery.listPage(pageSize * (pageNumber - 1), pageSize);
        Set<String> staterUsers = new HashSet<>();
        List<ResToBeProcessedDTO> resVos = taskList.stream().map(task -> {
            String nodeId = task.getTaskDefinitionKey();
            String instanceId = task.getProcessInstanceId();

            ProcessProgressVo instanceProgress = processService.getInstanceProgress(nodeId, instanceId);
            Map<String, Object> formData = instanceProgress.getFormData();
            String status = instanceProgress.getStatus();
            Boolean temporarySupplement = (Boolean) formData.get(ProjectDeclareConstants.BasicInformation.TEMPORARY_SUPPLEMENT);
            ResToBeProcessedDTO res = new ResToBeProcessedDTO();
            if (temporarySupplement.equals(param.getIsSupplement())) {
                ProcessDefinition processDef = repositoryService.createProcessDefinitionQuery()
                        .processDefinitionId(task.getProcessDefinitionId())
                        .singleResult();
                //从缓存取
                ProcessInstanceOwnerDto owner = runtimeService.getVariable(task.getExecutionId(), "owner", ProcessInstanceOwnerDto.class);
                ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(task.getProcessInstanceId()).singleResult();
                staterUsers.add(owner.getOwner());
                ProcessTaskVo processTaskVo = ProcessTaskVo.builder()
                        .taskId(task.getId())
                        .taskName(task.getName())
                        .taskDefKey(task.getTaskDefinitionKey())
                        .processDefId(task.getProcessDefinitionId())
                        .executionId(task.getExecutionId())
                        .nodeId(nodeId)
                        .deployId(processDef.getDeploymentId())
                        .processDefName(processDef.getName())
                        .version(processDef.getVersion())
                        .instanceId(instanceId)
                        .ownerId(processInstance.getStartUserId())
                        .ownerDeptId(owner.getOwnerDeptId())
                        .ownerDeptName(owner.getOwnerDeptName())
                        .createTime(processInstance.getStartTime())
                        .taskCreateTime(task.getCreateTime())
                        .build();
                res.setProcessTaskVo(processTaskVo);
                String projectName = (String) formData.get(ProjectDeclareConstants.BasicInformation.PROJECT_NAME);
                res.setProjectName(projectName);
                res.setReportUnitId(owner.getOwnerDeptId());
                res.setReportUnitName(owner.getOwnerDeptName());
                res.setReportAmount((Integer) formData.get(ProjectDeclareConstants.FundDeclareInfo.DECLARE_AMOUNT));
                res.setBudgetYear((Integer) formData.get(ProjectDeclareConstants.BasicInformation.BUDGET_YEAR));
                res.setProcessStatus(ProcessStatusEnum.getCodeByDesc(status));
                res.setProcessStatusName(status);
                res.setProcessLaunchTime(NdDateUtils.date2LocalDateTime(processInstance.getStartTime()));
            }
            return res;
        }).collect(Collectors.toList());
        List<ResToBeProcessedDTO> result = resVos.stream()
                .filter(r -> r.getProjectName().equals(param.getProjectName()))
                .filter(r -> r.getReportUnitName().equals(param.getReportUnitName()))
                .filter(r -> r.getBudgetYear().equals(param.getBudgetYear()))
                .filter(r -> r.getProcessLaunchTime().isBefore(param.getProcessLaunchStartTime()))
                .filter(r -> r.getProcessLaunchTime().isAfter(param.getProcessLaunchEndTime())).collect(Collectors.toList());
        //取用户信息，减少数据库查询，一次构建
        if (CollectionUtil.isNotEmpty(staterUsers)) {
            Map<String, OrgUser> userMap = userDeptOrLeaderService.getUserMapByIds(staterUsers);
            page.setRecords(result.stream().peek(v -> v.getProcessTaskVo().setOwner(userMap.get(v.getProcessTaskVo().getOwnerId()))).collect(Collectors.toList()));
        }
        return PageVo.of(resVos,page.getTotal());
    }


    public void exportProjectList(HttpServletResponse response, ReqToBeProcessedDTO param) {
        PageVo<ResToBeProcessedDTO> page =
                queryProjectList(param);
        List<ResToBeProcessedDTO> collect = (List<ResToBeProcessedDTO>) page.getRecords();
        String fileName = null;
        if (param.getIsSupplement()){
            fileName = "增补项目列表";
        }else {
            fileName = "非增补项目列表";
        }
        ExcelDownUtil.setFileName(fileName,response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), ResToBeProjectListExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
